"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var PrometheusModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Prometheus = exports.PrometheusModule = exports.MetricController = void 0;
const common_1 = require("@nestjs/common");
const Prometheus = require("prom-client");
exports.Prometheus = Prometheus;
const LabelsHelper_1 = require("./LabelsHelper");
function getTimeFrom(startAt) {
    const diff = process.hrtime(startAt);
    return diff[0] * 1e3 + diff[1] * 1e-6;
}
let MetricController = class MetricController {
    metrics() {
        return Prometheus.register.metrics();
    }
};
__decorate([
    (0, common_1.Get)('/metrics'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MetricController.prototype, "metrics", null);
MetricController = __decorate([
    (0, common_1.Controller)()
], MetricController);
exports.MetricController = MetricController;
let PrometheusModule = PrometheusModule_1 = class PrometheusModule {
    constructor() {
        this.PROM_LABELS = ['method', 'route', 'status'];
        this.httpRequestsTotal = new Prometheus.Counter({
            name: `http_requests_total`,
            help: 'Counter for total requests received',
            labelNames: this.PROM_LABELS,
        });
        this.httpRequestDurationSeconds = new Prometheus.Histogram({
            name: `http_request_duration_seconds`,
            help: 'Duration of HTTP requests in seconds',
            labelNames: this.PROM_LABELS,
            buckets: Prometheus.exponentialBuckets(0.05, 1.75, 8),
        });
        this.httpRequestLengthBytes = new Prometheus.Histogram({
            name: `http_request_length_bytes`,
            help: 'Content-Length of HTTP request',
            labelNames: this.PROM_LABELS,
            buckets: [],
        });
        this.httpResponseLengthBytes = new Prometheus.Histogram({
            name: `http_response_length_bytes`,
            help: 'Content-Length of HTTP request',
            labelNames: this.PROM_LABELS,
            buckets: [],
        });
        this.LabelsHelper = LabelsHelper_1.LabelsHelper;
    }
    configure(consumer) {
        consumer.apply(this.middleware()).forRoutes('*');
    }
    middleware() {
        return (req, res, next) => {
            const startAt = process.hrtime();
            next();
            res.on('finish', () => {
                const time = getTimeFrom(startAt);
                const labels = this.middlewareLabels(req, res);
                this.httpRequestsTotal.labels(...labels).inc();
                this.httpRequestDurationSeconds.labels(...labels).observe(time / 1000);
                this.httpRequestLengthBytes
                    .labels(...labels)
                    .observe(req.get('Content-Length') ? parseInt(req.get('Content-Length')) : 0);
                this.httpResponseLengthBytes
                    .labels(...labels)
                    .observe(res.get('Content-Length') ? parseInt(res.get('Content-Length')) : 0);
                this.customMetrics(labels, req, res, time);
            });
        };
    }
    middlewareLabels(request, response) {
        const method = LabelsHelper_1.LabelsHelper.method(request, response);
        const route = LabelsHelper_1.LabelsHelper.route(request, response);
        const status = LabelsHelper_1.LabelsHelper.statusCode(request, response);
        return [method, route, status];
    }
    customMetrics(labels, req, res, time) { }
    static fromRoot({ controllers = null } = {}) {
        return {
            controllers: controllers || [MetricController],
            module: PrometheusModule_1,
        };
    }
};
PrometheusModule = PrometheusModule_1 = __decorate([
    (0, common_1.Module)({}),
    __metadata("design:paramtypes", [])
], PrometheusModule);
exports.PrometheusModule = PrometheusModule;
